# XTA Client Beispielcode

Die KoSIT stellt Beispielcode für die Umsetzung des XTA Clients zur Verfügung, welcher aufbauend auf der Implementierung des XTA Clients durch die Behörde für Stadtentwicklung und Wohnen Hamburg, im Rahmen der digitalen Bauleitplanung (Projekt DiPlanung; Nachrichtenformat XBauleitplanung) des Amts für Landesplanung und Stadtentwicklung, weiterentwickelt wurde. Der Beispielcode für den XTA Client basiert auf der Spezifikation XTA 2 Version 3 und unterstützt Implementierende darin, die grundlegenden Funktionalitäten des XTA-WS Clients nachzuvollziehen.

Der Beispielcode wurde im Auftrag der KoSIT von init AG erstellt (Hauptentwickler ist Heiko Green). Die "init"-spezifischen Einträge im Code gelten als Platzhalter für den Bezeichner eines Fachverfahrens oder der anwendenden Firma die den Nachrichtentransport implementiert.

**Der vorliegende Beispielcode ist nicht normativ, normatives Dokument ist die Spezifikation des Standards XTA.** Eine kontinuierliche Weiterentwicklung dieses Hilfsmittels ist Stand Februar 2021 nicht vorgesehen, redaktionelle Verbesserungen und Fehlerkorrekturen werden jedoch nachgepflegt. Anregungen für Verbesserungen und Erweiterungen nimmt die KoSIT gerne unter kosit@finanzen.bremen.de entgegen.

##Änderungshistorie

| **Version** | **Datum** | **Änderungen**
|:---|:---|:---|
| 1.0 | 26.02.2021 | Veröffentlichung der ersten Version |
| 1.1 | 21.07.2021 | Abstraktion des Beispielcodes und weitere Verbesserungen <br> - alle Referenzen auf die spezifischen Nachrichtenbroker-/vermittler entfernt <br> - alle konkreten Referenzen auf spezifische Test-Server durch Platzhalter ersetzt <br> - alle Bezeichnungen „Nachrichtenbroker“ entfernt und in „Nachrichtenvermittler“ umbenannt <br> - Dateipfade/Pakete in "de/xoev/xta" umbenannt <br> - Projektdateien (.pom, .properties etc) mit Platzhaltern versehen <br> - weitere Erläuterungen/Hinweise im Code hinzugefügt |

## Zielsetzung und Hintergrund

Das vorliegende Dokument bietet einen kurzen Überblick über Funktionsumfang und Nutzung des XTA Java-Beispielcodes. Der Beispielcode kapselt grundlegende Funktionalitäten der XTA-Schnittstelle und vereinfacht dadurch deren Nutzung in Java-Implementierungen. Er wird mit Beispielen für Testfälle ausgeliefert, welche die Interaktion eines Clients mit einer Gegenstelle adressieren. Die beigefügten Testbeispiele mit Stand vom Dezember 2020 befassen sich mit zwei Gegenstellen (Nachrichtenvermittler A/XBauleitplanung und Nachrichtenvermittler B/xdomea Regierung).

## Einrichtung

Der Beispielcode und Testfälle liegen als Maven-Projekt (JDK: Java 1.8) vor. Zur Einrichtung sind neben der Installation noch folgende Schritte notwendig:

- Accounts beim Nachrichtenvermittler anlegen (bzw. anlegen lassen)
- Clientzertifikate ausstellen lassen
- Serverzertifikat anfordern/oder selbst vom Server laden

## Anwendungsbeispiele

Die folgenden Abschnitte zeigen jeweils anhand eines UML-Sequenzdiagramms sowie von Beispielcode auf, wie die XTA-Bibliothek zur Interaktion mit einer Gegenstelle genutzt werden kann.

### Versand einer Nachricht

**Schematische Darstellung (mermaid-Markdown):**

```mermaid
%% Versand einer Nachricht
  sequenceDiagram
    participant A as Client
    participant B as XTA-Client
    participant C as Nachrichtenvermittler
    Note left of A: (1) Nachricht erzeugen
    Note left of A: (2) SenderID erzeugen
    Note left of A: (3) XTA-Client erzeugen
    Note left of A: (4) Nachricht senden
    A->>B: sendMessage
    activate A
    activate B
    B->>C: checkAccountActive
    activate C
    C-->>B: 
    deactivate C
    B->>C: lookupService
    activate C
    C-->>B: 
    deactivate C
    B->>C: createMessageID
    activate C
    C-->>B: 
    deactivate C
    B->>C: sendMessage
    activate C
    deactivate C
    B->>C: getTransportReport
    activate C
    C-->>B: 
    deactivate C
    Note left of A: (5) Nachrichtenstatus prüfen
    B-->>A: 
	deactivate B
	deactivate A
```

**Beispielcode:**

```java
// (1) Nachricht erzeugen
final Message message = new Message("application/xml", IOUtils.toByteArray(new FileInputStream(fileName)))
// (2) die Author-Identität aus Settings erzeugen
XtaClientSettings xtaClientSettings_author = new XtaClientSettings(BaseTest.class.getClassLoader().getResourceAsStream("config/xtaclient_author.properties"))
Identifier senderId =  Identifier authorId = Identifier.from(xtaClientSettings_author);
// ... oder programmatisch:
// Identifier authorId = new Identifier("beispiel:456", "BEISPIEL");
// (3) XTA-Client mit Author-Identität (programmatisch) erzeugen
XtaClient authorClient = new XtaClient(XtaClientConfig
        .create(new URL(managementServiceUrl), 
                new URL(sendServiceUrl), 
                new URL(msgBoxServiceUrl), 
                authorId,
                clientCertKeystoreK1, 
                this.clientCertKeystorePassword, 
                this.trustCertKeystore, 
                this.trustCertKeystorePassword).build());
// DeliveryOptions erzeugen - Service: http://www.osci.de/xdomea23/xdomea23Reg2Reg.wsdl
DeliveryOptions deliveryOptions = new DeliveryOptions(serviceName)
deliveryOptions.withCheckDeliveryStatus(DeliveryStatus.RECEIVED_IN_MSGBOX);
// mit BusinessScenario: XDOMEAREG_DATA,
deliveryOptions.withBusinessScenario(businessScenario);
// (4) senden der Nachricht (an sich selbst)
final MessageTransportReport transportReport = authorClient.sendMessage(authorId, deliveryOptions, message);
final String messageId = transportReport.getMessageId();
// (5) Nachrichtenstatus prüfen
MessageTransportReport messageTransportReport = authorClient.getMessageTransportReport(messageId);
```


### Abfrage einer Nachricht

**Schematische Darstellung (mermaid-Markdown):**

```mermaid
%% Abfrage einer Nachricht
  sequenceDiagram
    participant A as Client
    participant B as XTA-Client
    participant C as Nachrichtenvermittler
    Note left of A: (1) EmpfängerID erzeugen
    Note left of A: (2) XTA-Client erzeugen
    Note left of A: (3) Postfach abfragen
    A->>B: getMessageTransportReportList
    activate A
    activate B
    B->>C: getStatusList
    activate C
    C-->>B: 
    deactivate C
    B->>C: getTransportReport
    activate C
    C-->>B: 
    deactivate C
    B->>A: 
	deactivate B
	Note left of A: (4) Nachricht abholen
    A->>B: getMessage
    activate B
    B->>C: getMessage
    activate C
    C-->>B: 
    deactivate C
    B->>C: getTransportReport
    activate C
    C-->>B: 
    deactivate C
    B->>A: 
	deactivate B
	Note left of A: ...Nachricht verarbeiten...
	Note left of A: (5) Nachricht schließen
    A->>B: close
    activate B
    B->>C: close
    activate C
    C-->>B: 
    deactivate C
    B->>A: 
	deactivate B
	deactivate A
```


**Beispielcode:**

``````java
// (1) Empfänger-Identität programmatisch per Konstruktor erzeugen
Identifier readerId = new Identifier("beispiel:789", "BEISPIEL");
// (2) XTA-Client mit Empfänger-Identität (programmatisch) erzeugen
XtaClient readerClient = new XtaClient(XtaClientConfig
    .create(new URL(managementServiceUrl), 
            new URL(sendServiceUrl), 
            new URL(msgBoxServiceUrl), 
            readerId,
            clientCertKeystoreA1, 
            this.clientCertKeystorePassword, 
            this.trustCertKeystore, 
            this.trustCertKeystorePassword).build());
// Nachrichtenselektor für Postfachabfrage erzeugen
MessageFilter messageFilter = new MessageFilter().newEntries().withCount(100).withTimeFrom(sendDate);
// (3) die eigentliche Postfachabfrage
List<MessageTransportReport> messageTransportReportList = readerClient.getMessageTransportReportList(messageFilter);
// relevante NachrichtenId aus messageTransportReportList ermitteln
// (je nach Nachrichtenselektor, können mehr als eine ID vorhanden sein - wir gehen hier nur von einer aus)
final String messageIdReceive = // ermittleID(messageTransportReportList)
// (4) Nachricht aus Postfach abholen
final GetMessageResult getMessageResult = readerClient.getMessage(messageIdReceive);
// ------------------------------------------------------------------------
// Nachricht zur weiteren Verarbeitung: getMessageResult.getMessage().getData()
// ------------------------------------------------------------------------
// (5) Nachricht schließen/quittieren
readerClient.close(messageIdReceive);
// hier könnte man nun mit dem Sender-Client den Nachrichtenstatus prüfen und sollte festellen das der
// Empfänger die Nachricht abgeholt bzw. quittiert hat:
// MessageTransportReport messageTransportReport = sender.getMessageTransportReport(messageId);
``````

## Abhängigkeiten

**Abhängigkeiten der XTA-Bibliothek:**

*CTX - Java-Webservice-Stack*:

- Core: https://mvnrepository.com/artifact/org.apache.cxf/cxf-core

*Utilities:*

- Sammlungen: https://mvnrepository.com/artifact/org.apache.commons/commons-collections4
- Beanutils: https://mvnrepository.com/artifact/org.jodd/jodd-bean
- Codegenutils: https://mvnrepository.com/artifact/org.projectlombok/lombok
- Input/Output: https://mvnrepository.com/artifact/commons-io/commons-io
- Diverses (zb. StringUtils): https://mvnrepository.com/artifact/org.apache.commons/commons-lang3

*Testing*:

- JUnit: https://mvnrepository.com/artifact/org.junit.jupiter/junit-jupiter-api
- Assert-Util: https://mvnrepository.com/artifact/org.assertj/assertj-core



**Abhängigkeiten der Testprojekte:**

Abhängigkeit zur XTA-Bibliothek selbst (und somit auch zu allen o.g. Komponenten)

*Testing:*

- JUnit: https://mvnrepository.com/artifact/org.junit.jupiter/junit-jupiter-api
- Assert-Util: https://mvnrepository.com/artifact/org.assertj/assertj-core
- XML Santuario: https://mvnrepository.com/artifact/org.apache.santuario/xmlsec

##Server Modus

Der XTA Beispielcode enthält eine exemplarische Server-Mode Implementierung bei denen die 
Funktionalitäten bzw. die API der Bibliothek als SOAP-Web-Service zur Verfügung gestellt wird.

Zu Starten ist hierbei der Server mittels: 

    java -jar <xta-client-artefakt.jar> -settings <settingsdatei> -port <port>
    
oder programmatisch:

    XtaClientSoapServer.main(new String[]{"-settings", <settingsdatei>, "-port", <port>});
    
Die SOAP-Webservice-Schnittstelle kann beispielweise mittels SOAPUI aufgerufen und getestet werden.    
    
Im Server-Mode sind folgende Funktionen der Client-Bibliothek integriert:
  
- close
- cancelMessage

Die programmatische Verwendung und der Aufruf der Server-Mode-Variante kann beispielhaft aus der 
Testklasse SoapServerTest.java entnommen werden.