/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.codes;

import de.xoev.xta.client.model.DeliveryOptions;
import lombok.Getter;

/**
 * Einige vordefinierte Werte für "business.scenario" (Fachkontext/Verfahren). Werte die nicht auftauchen können über den
 * Stringsetter gesetzt werden {@link DeliveryOptions#withBusinessScenario(String)}.
 *
 * @see <a href="https://www.xrepository.de/details/urn:de:xta:codeliste:business.scenario">business.scenario</a>
 */
@Getter
public enum BusinessScenario {

  /**
   * Fachkontext XBAULEITPLANUNG.
   */
  XBAULEITPLANUNG_DATA("XBAULEITPLANUNG_DATA"),

  /**
   * Fachkontext XDOMEAREG.
   */
  XDOMEAREG_DATA("XDOMEAREG_DATA"),

  /**
   * Fachkontext XInneres.
   */
  XINNERES_DATA("XINNERES_DATA"),

  /**
   * Fachkontext XAusländer.
   */
  XAUSLAENDER_DATA("XAUSLAENDER_DATA"),

  /**
   * Fachkontext OSCI-XMeld.
   */
  XMELD_DATA("XMELD_DATA"),

  /**
   * Fachkontext XPersonenstand.
   */
  XPERSONENSTAND_DATA("XPERSONENSTAND_DATA"),

  /**
   * Fachkontext XhD.
   */
  XHD_DATA("XHD_DATA"),

  /**
   * Fachkontext XGewerbeanzeige.
   */
  GEWERBE_DATA("GEWERBE_DATA");

  private String code;

  BusinessScenario(String code) {
    this.code = code;
  }
}
