/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.codes;

import de.xoev.xta.client.model.Identifier;
import lombok.Getter;

/**
 * Einige vordefinierte Werte für "type.of.party.identifier". Werte die nicht auftauchen können über den Stringsetter gestzt werden * {@link
 * Identifier (final String, String)}
 *
 * @see <a href"https://www.xrepository.de/details/urn:de:xta:codeliste:type.of.party.identifier">type.of.party.identifier</a>
 */
@Getter
public enum IdentifierType {

  /**
   * XÖV-DVDV-Infrastruktur: Identifizierungsschema gemäß DVDV-Behördenschlüssel	Nach den Regeln dieses Identifizierungsschemas
   * ist z.B. die Kennung "psw:01002110" aufgebaut.
   */
  XOEV("xoev"),

  /**
   * SAFE-Infrastruktur des Justizressorts: Identifizierungsschema gemäß SAFE Nach den Regeln dieses Identifizierungsschemas ist
   * z.B. die Kennung "safe-1363359638330-001060793" aufgebaut.
   */
  JUSTIZ("justiz");

  private String code;

  IdentifierType(String code) {
    this.code = code;
  }
}
