/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.config;

import de.xoev.xta.client.exception.ClientInitializationException;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Klasse zum Laden der Settings über eine Propertie-Datei.
 */
public final class XtaClientSettings {

  public final static String BROKER_MANAGEMENT_SERVICE_ENDPONIT = "broker.manangement-service.endpoint";
  public final static String BROKER_SEND_SERVICE_ENDPOINT = "broker.send-service.endpoint";
  public final static String BROKER_MSGBOX_SERVICE_ENDPOINT = "broker.msgbox-service.endpoint";
  public final static String BROKER_KEYSTORE_TRUSTCERT_PATH = "broker.keystore.trustcert.path";
  public final static String BROKER_KEYSTORE_TRUSTCERT_PASSPHRASE = "broker.keystore.trustcert.passphrase";

  public final static String CLIENT_KEYSTORE_CERT_PATH = "client.keystore.cert.path";
  public final static String CLIENT_KEYSTORE_CERT_PASSPHRASE = "client.keystore.cert.passphrase";

  public final static String CLIENT_IDENTIFIER_ID_KEY = "client.identifier.id";
  public final static String CLIENT_IDENTIFIER_TYPE_KEY = "client.identifier.type";
  public final static String CLIENT_IDENTIFIER_CATEGORY_KEY = "client.identifier.category";
  public final static String CLIENT_IDENTIFIER_NAME_KEY = "client.identifier.name";


  private Properties props = new Properties();

  public XtaClientSettings(final InputStream is) throws ClientInitializationException {
    try {
      props.load(is);
      if (props.size() == 0) {
        throw new ClientInitializationException("Properties sind nicht initialisiert.");
      }
    } catch (final IOException e) {
      throw new ClientInitializationException("Properties konnten nicht geladen werden.");
    }
  }

  public String get(String key) {
    return (String) props.get(key);
  }
}
