/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.core;

import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.cxf.binding.soap.interceptor.SoapPreProtocolOutInterceptor;
import org.apache.cxf.io.CachedOutputStream;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.AbstractPhaseInterceptor;
import org.apache.cxf.phase.Phase;

/**
 * Klasse um leere Namespace-Angaben zu entfernen.
 */
@Slf4j
class EmptyNamespaceMessageOutInterceptor extends AbstractPhaseInterceptor<Message> {

  public EmptyNamespaceMessageOutInterceptor() {
    super(Phase.PRE_STREAM);
    addBefore(SoapPreProtocolOutInterceptor.class.getName());
  }

  public void handleMessage(Message message) {

    OutputStream os = message.getContent(OutputStream.class);
    CachedStream cs = new CachedStream();
    message.setContent(OutputStream.class, cs);
    message.getInterceptorChain().doIntercept(message);

    try {
      cs.flush();
      closeQuietly(cs);
      CachedOutputStream csnew = (CachedOutputStream) message.getContent(OutputStream.class);

      String currentEnvelopeMessage = IOUtils
          .toString(csnew.getInputStream(), "UTF-8");

      csnew.flush();
      closeQuietly(csnew);

      if (log.isDebugEnabled()) {
        log.debug("Outbound message: " + currentEnvelopeMessage);
      }

      String res = changeOutboundMessage(currentEnvelopeMessage);
      if (res != null) {
        if (log.isDebugEnabled()) {
          log.debug("Outbound message has been changed: " + res);
        }
      }
      res = res != null ? res : currentEnvelopeMessage;

      InputStream replaceInStream = IOUtils
          .toInputStream(res, "UTF-8");

      IOUtils.copy(replaceInStream, os);
      replaceInStream.close();
      closeQuietly(replaceInStream);

      os.flush();
      message.setContent(OutputStream.class, os);
      closeQuietly(os);

    } catch (IOException ioe) {
      log.error("Unable to perform change.", ioe);
      throw new RuntimeException(ioe);
    }
  }

  protected String changeOutboundMessage(String currentEnvelope) {
    currentEnvelope = currentEnvelope.replace("xmlns=\"\"", "");
    return currentEnvelope;
  }

  private class CachedStream extends CachedOutputStream {

    public CachedStream() {
      super();
    }
  }

  // WA - closeQuietly aus IOUtils ist veraltet, daher hier eine eigene Iplementierung
  private static void closeQuietly(final Closeable closeable) {
    try {
      if (closeable != null) {
        closeable.close();
      }
    } catch (final IOException ioe) {
      // ignore
    }
  }
}