/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.core;

import de.xoev.xta.client.exception.ClientInitializationException;
import de.xoev.transport.xta._211.CancelDeniedException;
import de.xoev.transport.xta._211.InvalidMessageIDException;
import de.xoev.transport.xta._211.LookupServiceRequest;
import de.xoev.transport.xta._211.LookupServiceResponse;
import de.xoev.transport.xta._211.ManagementPortType;
import de.xoev.transport.xta._211.ParameterIsNotValidException;
import de.xoev.transport.xta._211.PermissionDeniedException;
import de.xoev.transport.xta._211.TransportReport;
import de.xoev.transport.xta._211.XTAService;
import de.xoev.transport.xta._211.XTAWSTechnicalProblemException;
import eu.osci.ws._2014._10.transport.PartyType;
import java.net.URL;
import lombok.extern.slf4j.Slf4j;
import org.w3._2005._08.addressing.AttributedURIType;

/**
 * gewrapte Serviceklasse für den ManagementServcice eines Brokers
 */
@Slf4j
public class ManagementServiceClient extends WsClient<ManagementPortType> {

  public static ManagementServiceClientBuilder builder(final URL serviceUrl) {
    return new ManagementServiceClientBuilder(serviceUrl);
  }

  public ManagementServiceClient(final WsClientConfig conf) throws ClientInitializationException {
    this.clientConfiguration = conf;
    ManagementServiceClient.log.info(VersionInfo.getInfo());
    ManagementServiceClient.log.info(this.clientConfiguration.toString());
    this.initializeService();
  }

  @Override
  void initializePort(final URL url) {
    final XTAService service = new XTAService(url);
    port = service.getManagementPort();
  }

  public void checkAccountActive(final PartyType authorIdentifier)
      throws PermissionDeniedException, XTAWSTechnicalProblemException {
    port.checkAccountActive(authorIdentifier);
  }

  public TransportReport getTransportReport(final AttributedURIType messageID, final PartyType authorIdentifier)
      throws PermissionDeniedException, XTAWSTechnicalProblemException, InvalidMessageIDException {
    return port.getTransportReport(messageID, authorIdentifier);
  }

  public void cancelMessage(final AttributedURIType messageID, final PartyType authorIdentifier)
      throws XTAWSTechnicalProblemException, PermissionDeniedException, CancelDeniedException, ParameterIsNotValidException {
    port.cancelMessage(messageID, authorIdentifier);
  }

  public LookupServiceResponse lookupService(final LookupServiceRequest lookupServiceRequest, final PartyType authorIdentifier)
      throws ParameterIsNotValidException, PermissionDeniedException, XTAWSTechnicalProblemException {
    return port.lookupService(lookupServiceRequest, authorIdentifier);
  }

  public AttributedURIType createMessageId(final PartyType authorIdentifier)
      throws PermissionDeniedException, XTAWSTechnicalProblemException {
    return port.createMessageId(authorIdentifier);
  }
}
