/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.core;

import de.xoev.transport.xta._211.*;
import de.xoev.xta.client.exception.ClientInitializationException;
import eu.osci.ws._2008._05.transport.*;
import eu.osci.ws._2014._10.transport.MessageMetaData;
import eu.osci.ws._2014._10.transport.PartyType;
import lombok.extern.slf4j.Slf4j;

import javax.xml.ws.Holder;
import java.net.URL;

/**
 * gewrapte Serviceklasse für den MsgBoxServcice eines Brokers
 */
@Slf4j
public class MsgBoxServiceClient extends WsClient<MsgBoxPortType> {

  public static MsgBoxServiceClientBuilder builder(final URL serviceUrl) {
    return new MsgBoxServiceClientBuilder(serviceUrl);
  }

  public MsgBoxServiceClient(final WsClientConfig conf) throws ClientInitializationException {

    this.clientConfiguration = conf;
    MsgBoxServiceClient.log.info(VersionInfo.getInfo());
    MsgBoxServiceClient.log.info(this.clientConfiguration.toString());
    this.initializeService();
  }

  @Override
  void initializePort(final URL url) {
    final XTAService service = new XTAService(url);
    port = service.getMsgBoxPort();
  }

  public GenericContentContainer getMessage(final MsgBoxFetchRequest fetchRequest, final PartyType authorIdentifier,
      final Holder<MessageMetaData> messageMetaData, final Holder<MsgBoxResponseType> fetchResponseHeader)
      throws InvalidMessageIDException, PermissionDeniedException, XTAWSTechnicalProblemException {
    return port.getMessage(fetchRequest, authorIdentifier, messageMetaData, fetchResponseHeader);
  }

  public MsgStatusListType getStatusList(final MsgBoxStatusListRequestType fetchRequest, final PartyType authorIdentifier,
      final Holder<MsgBoxResponseType> fetchResponseHeader) throws PermissionDeniedException, XTAWSTechnicalProblemException {
    return port.getStatusList(fetchRequest, authorIdentifier, fetchResponseHeader);
  }

  public GenericContentContainer getNextMessage(final MsgBoxGetNextRequestType fetchRequest, final PartyType authorIdentifier,
      final Holder<MessageMetaData> messageMetaData, final Holder<MsgBoxResponseType> fetchResponseHeader)
      throws InvalidMessageIDException, PermissionDeniedException, XTAWSTechnicalProblemException {
    return port.getNextMessage(fetchRequest, authorIdentifier, messageMetaData, fetchResponseHeader);
  }

  public MsgStatusListType getNextStatusList(final MsgBoxGetNextRequestType fetchRequest, final PartyType authorIdentifier,
      final Holder<MsgBoxResponseType> fetchResponseHeader) throws PermissionDeniedException, XTAWSTechnicalProblemException {
    return port.getNextStatusList(fetchRequest, authorIdentifier, fetchResponseHeader);
  }

  public void close(final MsgBoxCloseRequestType fetchRequest, final PartyType authorIdentifier)
      throws InvalidMessageIDException, PermissionDeniedException, XTAWSTechnicalProblemException {
    port.close(fetchRequest, authorIdentifier);
  }
}
