/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.core;

import de.xoev.xta.client.exception.ClientInitializationException;
import java.net.URL;

import lombok.AccessLevel;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

/**
 * Konstruiert eine Instanz des {@link WsClient}.
 */
@Getter(AccessLevel.PACKAGE)
@Slf4j
public class MsgBoxServiceClientBuilder extends ServiceBuilder<MsgBoxServiceClient> {

  /**
   * Initialisiert einen {@link ServiceBuilder} mit ServiceUrl-Host und ServiceUrl-Port.
   *
   * @param serviceUrl URL des Endpoints
   */
  MsgBoxServiceClientBuilder(final URL serviceUrl) {
    super(serviceUrl);
  }

  /**
   * Erstellt eine Instanz vom Typ AnlieferungServiceClient
   *
   * @return ManagementServiceClient
   * @throws ClientInitializationException {@link ClientInitializationException}
   */
  @Override
  public MsgBoxServiceClient build() throws ClientInitializationException {
    this.validateConfiguration();
    return new MsgBoxServiceClient(this.clientConfiguration);
  }
}
