/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.core;

import de.xoev.xta.client.exception.ClientInitializationException;
import de.xoev.transport.xta._211.GenericContentContainer;
import de.xoev.transport.xta._211.MessageSchemaViolationException;
import de.xoev.transport.xta._211.MessageVirusDetectionException;
import de.xoev.transport.xta._211.ParameterIsNotValidException;
import de.xoev.transport.xta._211.PermissionDeniedException;
import de.xoev.transport.xta._211.SendPortType;
import de.xoev.transport.xta._211.SyncAsyncException;
import de.xoev.transport.xta._211.XTAService;
import de.xoev.transport.xta._211.XTAWSTechnicalProblemException;
import eu.osci.ws._2008._05.transport.X509TokenContainerType;
import eu.osci.ws._2014._10.transport.MessageMetaData;
import java.net.URL;
import javax.xml.ws.Holder;
import lombok.extern.slf4j.Slf4j;

/**
 * gewrapte Serviceklasse für den SendServcice eines Brokers
 */
@Slf4j
public class SendServiceClient extends WsClient<SendPortType> {

  public static SendServiceClientBuilder builder(final URL serviceUrl) {
    return new SendServiceClientBuilder(serviceUrl);
  }

  public SendServiceClient(final WsClientConfig conf) throws ClientInitializationException {
    this.clientConfiguration = conf;
    SendServiceClient.log.info(VersionInfo.getInfo());
    SendServiceClient.log.info(this.clientConfiguration.toString());
    this.initializeService();
  }

  @Override
  void initializePort(final URL url) {
    final XTAService service = new XTAService(url);
    port = service.getSendXtaPort();
  }

  public void sendMessage(final GenericContentContainer genericContainer, final MessageMetaData messageMetaData,
      final X509TokenContainerType x509TokenContainer)
      throws PermissionDeniedException, XTAWSTechnicalProblemException, MessageSchemaViolationException, MessageVirusDetectionException, ParameterIsNotValidException, SyncAsyncException {
    port.sendMessage(genericContainer, messageMetaData, x509TokenContainer);
  }

  public void sendMessageSync(final Holder<GenericContentContainer> genericContainer,
      final Holder<MessageMetaData> messageMetaData, final Holder<X509TokenContainerType> x509TokenContainer)
      throws PermissionDeniedException, XTAWSTechnicalProblemException, MessageSchemaViolationException, MessageVirusDetectionException, ParameterIsNotValidException, SyncAsyncException {
    port.sendMessageSync(genericContainer, messageMetaData, x509TokenContainer);
  }
}
