/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.core;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * Klasse mit VersionInfo
 */
@Slf4j
@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
public final class VersionInfo {

  private static String build;

  private static String buildTimestamp;

  private static String version;

  public static String getInfo() {
    return String.format("VersionInfo: version: %s, build: %s, timestamp: %s", VersionInfo.version, VersionInfo.build,
        VersionInfo.buildTimestamp);
  }

  static {
    final Properties p = new Properties();
    try (final InputStream in = VersionInfo.class.getClassLoader().getResourceAsStream("version.properties")) {

      p.load(in);

      VersionInfo.build = p.getProperty("bamboo_buildNumber");
      VersionInfo.buildTimestamp = p.getProperty("build_creation_timestamp");
      VersionInfo.version = p.getProperty("project_version");

    } catch (final IOException e) {
      log.error(e.getMessage(), e);
      throw new IllegalStateException("Can not initialize version information");
    }
  }
}
