/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.mapping;

import de.xoev.xta.client.exception.ClientException;
import de.xoev.xta.client.exception.ClientInitializationException;
import de.xoev.xta.client.model.DeliveryOptions;
import de.xoev.xta.client.model.Identifier;
import de.xoev.xta.client.model.Identifier.CommunicationRole;
import de.xoev.xta.client.model.Message;
import de.xoev.xta.client.util.XmlUtil;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.lang3.StringUtils;

/**
 * Klasse um XTAClient Model-Objekte über ein Mapping aus dem Message-Objekt zu befüllen.
 */
public class MessageMapping {

  private Properties props = new Properties();
  private Message message;

  public MessageMapping(final Message message, final InputStream is) throws ClientInitializationException {
    try {
      this.message = message;
      this.props.load(is);
      if (this.props.size() == 0) {
        throw new ClientInitializationException("Properties sind nicht initialisiert.");
      }
    } catch (final IOException e) {
      throw new ClientInitializationException("Properties konnten nicht geladen werden.");
    }
  }

  private Map<String, String> getAllWithPrefix(String prefix) throws ClientInitializationException {
    final Map ret = new HashMap<>();
    for (Map.Entry p : props.entrySet()) {
      if (p.getKey().toString().startsWith(prefix)) {
        String key = p.getKey().toString();
        key = StringUtils.replace(key, prefix + ".", "");
        ret.put(key, p.getValue().toString());
      }
    }
    return ret;
  }

  public DeliveryOptions getDeliveryOptions() throws ClientException {
    return getDeliveryOptions(new DeliveryOptions());
  }

  public DeliveryOptions getDeliveryOptions(final DeliveryOptions deliveryOptions) throws ClientException {
    final Map<String, String> mappings = getAllWithPrefix("mapping.DeliveryOptions");
    return XmlUtil.transferMessageDataIntoBean(message.getDataAsString(), deliveryOptions, mappings);
  }

  public Identifier getIdentifier(final CommunicationRole forRole) throws ClientException {
    return getIdentifier(new Identifier(), forRole);
  }

  public Identifier getIdentifier(final Identifier identifier, final CommunicationRole forRole) throws ClientException {
    final Map<String, String> mappings = getAllWithPrefix("mapping.Identifier." + forRole);
    return XmlUtil.transferMessageDataIntoBean(message.getDataAsString(), identifier, mappings);
  }

}
