/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.model;

import java.math.BigInteger;

/**
 * Model-Klasse für Anhänge.
 */
public class Attachment extends Content {

  /**
   * Erzeugt einen Anhang aus Daten und Inhaltstyp.
   *
   * @param contentType Inhaltstyp
   * @param data        Inhalt als Byte-Array
   */
  public Attachment(final String contentType, final byte[] data) {
    super(contentType, data);
  }

  /**
   * Erzeugt einen Anhang aus Allem.
   *
   * @param contentType
   * @param data
   * @param contentDescription
   * @param encoding
   * @param filename
   * @param id
   * @param lang
   * @param size
   */
  public Attachment(final String contentType, final byte[] data, final String contentDescription, final String encoding,
      final String filename, final String id, final String lang, final BigInteger size) {

    this(contentType, data);

    setContentDescription(contentDescription);
    setEncoding(encoding);
    setFilename(filename);
    setId(id);
    setLanguage(lang);
    setSize(size);
  }
}
