/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.model;

import java.util.Date;
import lombok.Getter;
import org.apache.commons.lang3.ArrayUtils;

/**
 * Model-Klasse, repräsentiert eine Postfachabfrage. Es können unterschiedlich Kriterien gesetzt werden um eine Menge an
 * Nachrichten im Postfach zu selektieren.
 */
@Getter
public class MessageFilter {

  private int count = 10;
  private boolean newEntries = false;
  private Date timeFrom = null;
  private Date timeTo = null;
  private String messageId[] = null;

  /**
   * Anzahl der Nachricht festlegen.
   *
   * @param count die Anzahl
   * @return this
   */
  public MessageFilter withCount(final int count) {
    this.count = count;
    return this;
  }

  /**
   * Nur neue Nachrichten?
   *
   * @return this
   */
  public MessageFilter newEntries() {
    this.newEntries = true;
    return this;
  }

  /**
   * Alle Nachrichten die seit dem gegebenen Datum im Postfach sind.
   *
   * @param date das Datum
   * @return this
   */
  public MessageFilter withTimeFrom(final Date date) {
    this.timeFrom = date;
    return this;
  }

  /**
   * Alle Nachrichten die bis zum gegebenen Datum im Postfach angekommen sind.
   *
   * @param date das Datum
   * @return this
   */
  public MessageFilter withTimeTo(final Date date) {
    this.timeTo = date;
    return this;
  }

  /**
   * Angabe von NachrichtenID's zur explizieten Auswahl von bestimmten Nachrichten.
   *
   * @param messageId
   * @return
   */
  public MessageFilter withMessageIds(final String... messageId) {
    if (ArrayUtils.isEmpty(messageId)) {
      throw new IllegalArgumentException("one messageId is mandatory.");
    }

    this.messageId = messageId;
    return this;
  }
}
