/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.model;

import de.xoev.transport.xta._211.TransportReport;
import lombok.Getter;
import lombok.RequiredArgsConstructor;

/**
 * Model-Klasse die einen Nachrichten-Report repräsentiert. Intern wird XTA-TransportReport verwendet.
 */
@Getter
@RequiredArgsConstructor
public class MessageTransportReport {

  private final String messageId;
  private final TransportReport xtaTransportReport;

  public DeliveryStatus getDeliveryStatus() {
    if (xtaTransportReport != null) {
      if (xtaTransportReport.getMessageMetaData().getDeliveryAttributes().getReception() != null) {
        return DeliveryStatus.RECEIPT_FROM_MSGBOX;
      } else if (xtaTransportReport.getMessageMetaData().getDeliveryAttributes().getInitialFetch() != null) {
        return DeliveryStatus.INITIAL_FETCH_FROM_MSGBOX;
      } else if (xtaTransportReport.getMessageMetaData().getDeliveryAttributes().getDelivery() != null) {
        return DeliveryStatus.RECEIVED_IN_MSGBOX;
      } else if (xtaTransportReport.getMessageMetaData().getDeliveryAttributes().getInitialSend() != null) {
        return DeliveryStatus.SEND_TO_MSGBOX;
      }
    }

    return DeliveryStatus.UNKNOWN;
  }

  public MessageStatus getMessageStatus() {
    if (xtaTransportReport != null) {
      return MessageStatus.findByValue(xtaTransportReport.getMessageStatus().getStatus().intValue());
    }

    return MessageStatus.OPEN;
  }
}
