/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.modultest;


import static org.assertj.core.api.Assertions.assertThat;

import de.xoev.xta.client.XtaClient;
import de.xoev.xta.client.XtaClientConfig;
import de.xoev.xta.client.codes.IdentifierType;
import de.xoev.xta.client.exception.ClientInitializationException;
import de.xoev.xta.client.model.Attachment;
import de.xoev.xta.client.model.DeliveryStatus;
import de.xoev.xta.client.model.Identifier;
import de.xoev.xta.client.model.Message;
import de.xoev.xta.client.model.MessageTransportReport;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.assertj.core.api.Assertions;
import org.junit.Before;

/**
 * Basis-Testklasse.
 */
public abstract class BaseTest {

  final String managementServiceUrl = "https://../managementPort.svc";
  final String sendServiceUrl = "https://../sendPort.svc";
  final String msgBoxServiceUrl = "https://../msgBoxPort.svc";

  final String clientCertKeystore = "certs/client.jceks";
  final String clientCertKeystorePassword = "*pass*";
  final String trustCertKeystore = "certs/trust.jceks";
  final String trustCertKeystorePassword = "*pass*";

  Identifier author;
  Identifier reader;

  final String service = "urn:xbauleitplanung-planung2beteiligung:1.0";

  final int MAX_CHECK_ATTEMPS = 5;
  final long WAIT_TIME_BETWEEN_CHECKS = 15000;

  XtaClient client = null;

  @Before
  public void initialization() throws MalformedURLException, ClientInitializationException {
    this.author = new Identifier("diplanfhh:0200", IdentifierType.XOEV);
    this.reader = new Identifier("diplanfhh:0200", IdentifierType.XOEV);
    client = new XtaClient(XtaClientConfig
        .create(new URL(this.managementServiceUrl), new URL(this.sendServiceUrl), new URL(this.msgBoxServiceUrl), this.author,
            this.clientCertKeystore, this.clientCertKeystorePassword, this.trustCertKeystore, this.trustCertKeystorePassword)
        .withSoapLogging().withValidation()
        .build());

    Assertions.assertThat(client).isNotNull();
  }


  Message createTestMessage() throws IOException {
    return new Message("application/xml", IOUtils
        .toByteArray(this.getClass().getClassLoader().getResourceAsStream("data/planung2Beteiligung.BeteiligungNeu.0401.xml")));
  }

  List<Attachment> createTestAttachments() throws IOException {
    final List<Attachment> attachmentList = new ArrayList<>();
    for (int i = 0; i < 5; i++) {
      attachmentList.add(new Attachment("application/pdf", IOUtils
          .toByteArray(this.getClass().getClassLoader().getResourceAsStream("data/test.pdf"))));
    }
    return attachmentList;
  }

  void checkDeliveryStatus(final String messageId, final DeliveryStatus status) throws Exception {
    for (int i = 0; i < MAX_CHECK_ATTEMPS; i++) {
      final MessageTransportReport messageTransportReport = client.getMessageTransportReport(messageId);
      assertThat(messageTransportReport).isNotNull();
      if (messageTransportReport.getDeliveryStatus().ordinal() >= status.ordinal()) {
        return;
      }
      Thread.sleep(60 * 1000);
    }

    assertThat(true).isFalse();
  }
}
