/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.modultest;


import static org.assertj.core.api.Assertions.assertThat;

import de.xoev.xta.client.codes.BusinessScenario;
import de.xoev.xta.client.model.Attachment;
import de.xoev.xta.client.model.DeliveryOptions;
import de.xoev.xta.client.model.DeliveryStatus;
import de.xoev.xta.client.model.GetMessageResult;
import de.xoev.xta.client.model.Message;
import de.xoev.xta.client.model.MessageFilter;
import de.xoev.xta.client.model.MessageTransportReport;
import java.util.Date;
import java.util.List;

import org.junit.Ignore;
import org.junit.Test;

/**
 * Testklasse für Rolle Reader
 */
@Ignore("Einrichtung notwendig")
public class XtaClientReceiverTest extends BaseTest {


  @Test
  public void getMessageTransportReportList() throws Exception {
    final Message message = createTestMessage();
    final Date sendDate = new Date();
    MessageTransportReport transportReport = client
        .sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), message);
    final String messageId = transportReport.getMessageId();
    assertThat(messageId).isNotEmpty();
    checkDeliveryStatus(messageId, DeliveryStatus.RECEIVED_IN_MSGBOX);

    boolean rcFound = false;
    for (int i = 0; i < MAX_CHECK_ATTEMPS; i++) {
      List<MessageTransportReport> transportReports = client
          .getMessageTransportReportList(new MessageFilter().newEntries().withCount(500).withTimeFrom(sendDate));
      for (final MessageTransportReport messageTransportReport : transportReports) {
        if (messageTransportReport.getMessageId().equals(messageId)) {
          rcFound = true;
          break;
        }
      }
      if (rcFound) {
        break;
      }

      Thread.sleep(WAIT_TIME_BETWEEN_CHECKS);
    }

    assertThat(rcFound).isTrue();
  }

  @Test
  public void getMessageTest() throws Exception {
    final Message message = createTestMessage();
    MessageTransportReport transportReport = client
        .sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), message);
    final String messageId = transportReport.getMessageId();
    assertThat(messageId).isNotEmpty();
    checkDeliveryStatus(messageId, DeliveryStatus.RECEIVED_IN_MSGBOX);
    final GetMessageResult getMessageResult = client.getMessage(messageId);
    assertThat(getMessageResult).isNotNull();
    checkDeliveryStatus(messageId, DeliveryStatus.INITIAL_FETCH_FROM_MSGBOX);
    assertThat(getMessageResult.getMessage().getData()).isEqualTo(message.getData());
  }

  @Test
  public void getMessageAndCloseTest() throws Exception {
    final Message message = createTestMessage();
    MessageTransportReport transportReport = client
        .sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), message);
    final String messageId = transportReport.getMessageId();
    assertThat(messageId).isNotEmpty();
    checkDeliveryStatus(messageId, DeliveryStatus.RECEIVED_IN_MSGBOX);
    final GetMessageResult getMessageResult = client.getMessageAndClose(messageId);
    assertThat(getMessageResult).isNotNull();
    checkDeliveryStatus(messageId, DeliveryStatus.RECEIPT_FROM_MSGBOX);
    assertThat(getMessageResult.getMessage().getData()).isEqualTo(message.getData());
  }


  @Test
  public void getMessageWithAttachmentsTest() throws Exception {
    final Message message = createTestMessage();
    final List<Attachment> attachmentList = createTestAttachments();

    MessageTransportReport transportReport = client
        .sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), message, attachmentList);

    final String messageId = transportReport.getMessageId();
    assertThat(messageId).isNotEmpty();
    checkDeliveryStatus(messageId, DeliveryStatus.RECEIVED_IN_MSGBOX);

    final GetMessageResult getMessageResult = client.getMessage(messageId);
    assertThat(getMessageResult).isNotNull();
    assertThat(getMessageResult.getMessage().getData()).isEqualTo(message.getData());
    assertThat(getMessageResult.getAttachmentList().size()).isEqualTo(5);

    for (int i = 0; i < 5; i++) {
      assertThat(attachmentList.get(i).getData().equals(getMessageResult.getAttachmentList().get(i).getData()));
    }
  }

  @Test
  public void closeTest() throws Exception {
    final Message message = createTestMessage();
    MessageTransportReport transportReport = client
        .sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), message);
    final String messageId = transportReport.getMessageId();
    checkDeliveryStatus(messageId, DeliveryStatus.RECEIVED_IN_MSGBOX);
    final GetMessageResult getMessageResult = client.getMessage(messageId);
    client.close(messageId, getMessageResult.getRequestHandle());
    checkDeliveryStatus(messageId, DeliveryStatus.RECEIPT_FROM_MSGBOX);
  }


  @Test
  public void closeTestAll() throws Exception {
    for (int i = 0; i < 5; i++) {
      client.sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), createTestMessage());
    }
    List<MessageTransportReport> reports = client.getMessageTransportReportList(new MessageFilter().withCount(500));
    assertThat(reports.size()).isGreaterThan(4);
    for (final MessageTransportReport report : reports) {
      final GetMessageResult getMessageResult = client.getMessage(report.getMessageId());
      client.close(report.getMessageId(), getMessageResult.getRequestHandle());
    }
    assertThat(client.getMessageTransportReportList(new MessageFilter().withCount(500)).size()).isEqualTo(0);
  }
}
