/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.modultest;

import static org.assertj.core.api.Assertions.assertThat;

import de.xoev.xta.client.codes.BusinessScenario;
import de.xoev.xta.client.model.Attachment;
import de.xoev.xta.client.model.DeliveryOptions;
import de.xoev.xta.client.model.DeliveryStatus;
import de.xoev.xta.client.model.Message;
import de.xoev.xta.client.model.MessageStatus;
import de.xoev.xta.client.model.MessageTransportReport;
import de.xoev.transport.xta._211.CancelDeniedException;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.time.DateUtils;
import org.junit.Ignore;
import org.junit.Test;

/**
 * Testklasse für Rolle Author
 */
@Ignore("Einrichtung notwendig")
public class XtaClientSenderTest extends BaseTest {


  @Test
  public void sendMessageTestWithAttachments() throws Exception {
    final Message message = createTestMessage();
    final List<Attachment> attachmentList = createTestAttachments();

    MessageTransportReport transportReport = client
        .sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), message, attachmentList);
    assertThat(transportReport).isNotNull();
    assertThat(transportReport.getMessageId()).isNotEmpty();
    assertThat(transportReport.getMessageStatus()).isIn(MessageStatus.OPEN, MessageStatus.YELLOW, MessageStatus.GREEN);
    // assertThat(transportReport.getDeliveryStatus()).isNotIn(DeliveryStatus.RECEIVED_IN_MSGBOX);
    checkDeliveryStatus(transportReport.getMessageId(), DeliveryStatus.RECEIVED_IN_MSGBOX);
  }

  @Test
  public void sendMessageWithDeliveryStatusCheckTest() throws Exception {
    final Message message = createTestMessage();

    final DeliveryOptions options = new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA)
        .withCheckDeliveryStatus(DeliveryStatus.RECEIVED_IN_MSGBOX);
    MessageTransportReport transportReport = client.sendMessage(this.reader, options, message);
    assertThat(transportReport).isNotNull();
    assertThat(transportReport.getMessageId()).isNotEmpty();
    assertThat(transportReport.getMessageStatus()).isIn(MessageStatus.OPEN, MessageStatus.YELLOW, MessageStatus.GREEN);
    assertThat(transportReport.getDeliveryStatus()).isIn(DeliveryStatus.RECEIVED_IN_MSGBOX);
  }


  @Ignore //Abnhängig vom Übertragsintervall des Nachrichtenvermittlers
  @Test
  public void sendMessageDelayedTest() throws Exception {
    final Message message = createTestMessage();
    final DeliveryOptions deliveryOptions = new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA);
    Date date = new Date();
    date = DateUtils.addMinutes(date, 1);
    deliveryOptions.setSendNotBefore(date);
    MessageTransportReport transportReport = client.sendMessage(this.reader, deliveryOptions, message);
    assertThat(transportReport).isNotNull();
    assertThat(transportReport.getMessageId()).isNotEmpty();
    assertThat(transportReport.getMessageStatus()).isIn(MessageStatus.OPEN, MessageStatus.YELLOW, MessageStatus.GREEN);
    assertThat(transportReport.getDeliveryStatus()).isNotIn(DeliveryStatus.RECEIVED_IN_MSGBOX);
    checkDeliveryStatus(transportReport.getMessageId(), DeliveryStatus.RECEIVED_IN_MSGBOX);
  }


  @Test
  public void getMessageTransportReportTest() throws Exception {
    final Message message = createTestMessage();

    MessageTransportReport transportReport = client
        .sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), message);
    assertThat(transportReport).isNotNull();
    MessageTransportReport report = client.getMessageTransportReport(transportReport.getMessageId());
    assertThat(report).isNotNull();
    assertThat(report.getMessageId()).isEqualTo(transportReport.getMessageId());
  }

  @Test
  public void cancelMessageFailedTest() throws Exception {
    final Message message = createTestMessage();
    MessageTransportReport transportReport = client
        .sendMessage(this.reader, new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA), message);
    assertThat(transportReport).isNotNull();
    try {
      client.cancelMessage(transportReport.getMessageId());
      assertThat(true).isFalse();
    } catch (final Exception e) {
      assertThat(e instanceof CancelDeniedException).isTrue();
      assertThat(((CancelDeniedException) e).getFaultInfo().getErrorCode().getName())
          .contains("Der durch den Schalter NotBefore gesetzte Termin ist verstrichen.");
    }
  }

  @Test
  public void cancelMessageOkTest() throws Exception {
    final Message message = createTestMessage();
    final DeliveryOptions deliveryOptions = new DeliveryOptions(service, BusinessScenario.XBAULEITPLANUNG_DATA);
    Date date = new Date();
    date = DateUtils.addMinutes(date, 5);
    deliveryOptions.setSendNotBefore(date);
    MessageTransportReport transportReport = client.sendMessage(this.reader, deliveryOptions, message);
    assertThat(transportReport).isNotNull();
    client.cancelMessage(transportReport.getMessageId());
    assertThat(true).isTrue();
  }
}
