/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client;

import static org.assertj.core.api.Assertions.assertThat;

import de.xoev.xta.client.codes.IdentifierType;
import de.xoev.xta.client.exception.ClientInitializationException;
import de.xoev.xta.client.model.DeliveryStatus;
import de.xoev.xta.client.model.Identifier;
import de.xoev.xta.client.model.Message;
import de.xoev.xta.client.model.MessageTransportReport;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.junit.Before;
import org.junit.Ignore;

/**
 * Basis Testklasse
 */
@Ignore("Einrichtung notwendig")
public class BaseTest {

  final String managementServiceUrl = "https://.../managementPort.svc";
  final String sendServiceUrl = "https://.../sendPort.svc";
  final String msgBoxServiceUrl = "https://.../msgBoxPort.svc";

  final Identifier identifierK1 = new Identifier("ident:0007", IdentifierType.XOEV);
  final Identifier identifierA1 = new Identifier("ident:0007", IdentifierType.XOEV);

  final String clientCertKeystoreK1 = "certs/...jceks";
  final String clientCertKeystoreA1 = "certs/...jceks";
  final String clientCertKeystorePassword = "pass";

  final String trustCertKeystore = "certs/....jceks";
  final String trustCertKeystorePassword = "pass";

  XtaClient k1XtaClient;
  XtaClient a1XtaClient;

  public BaseTest() throws ClientInitializationException {

  }

  @Before
  public void initSenderReaderClient() throws ClientInitializationException, MalformedURLException {

    //K1 und A1 initiatlisieren
    k1XtaClient = new XtaClient(XtaClientConfig
        .create(new URL(managementServiceUrl), new URL(sendServiceUrl), new URL(msgBoxServiceUrl), identifierK1,
            clientCertKeystoreK1, this.clientCertKeystorePassword, this.trustCertKeystore, this.trustCertKeystorePassword)
        .withSoapLogging()
        .withValidation()
        .build());

    a1XtaClient = new XtaClient(XtaClientConfig
        .create(new URL(managementServiceUrl), new URL(sendServiceUrl), new URL(msgBoxServiceUrl), identifierA1,
            clientCertKeystoreA1, this.clientCertKeystorePassword, this.trustCertKeystore, this.trustCertKeystorePassword)
        .withSoapLogging()
        .withValidation()
        .build());
  }


  Message createMessage(final String fileName) throws IOException {
    return new Message("application/xml", IOUtils
        .toByteArray(this.getClass().getClassLoader().getResourceAsStream("data/" + fileName)));
  }

  void checkDeliveryStatus(final XtaClient client, final String messageId, final DeliveryStatus status) throws Exception {

    for (int i = 0; i < 5; i++) {
      final MessageTransportReport messageTransportReport = client.getMessageTransportReport(messageId);
      assertThat(messageTransportReport).isNotNull();
      if (messageTransportReport.getDeliveryStatus().ordinal() >= status.ordinal()) {
        return;
      }
      Thread.sleep(30 * 1000);
    }

    assertThat(true).isFalse();
  }

  String getAndCheckMessageInMessageTransportList(String k1MessageId, List<MessageTransportReport> messageTransportReportList) {
    String a1MessageId = null;
    for (final MessageTransportReport messageTransportReport : messageTransportReportList) {
      if (messageTransportReport.getMessageId().equals(k1MessageId)) {
        a1MessageId = messageTransportReport.getMessageId();
        break;
      }
    }
    assertThat(a1MessageId).isNotEmpty();
    return a1MessageId;
  }
}
