/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client;

import static org.assertj.core.api.Assertions.assertThat;

import de.xoev.xta.client.codes.BusinessScenario;
import de.xoev.xta.client.exception.ClientInitializationException;
import de.xoev.xta.client.model.DeliveryOptions;
import de.xoev.xta.client.model.DeliveryStatus;
import de.xoev.xta.client.model.GetMessageResult;
import de.xoev.xta.client.model.Message;
import de.xoev.xta.client.model.MessageFilter;
import de.xoev.xta.client.model.MessageTransportReport;
import java.util.Date;
import java.util.List;

import org.junit.Ignore;
import org.junit.Test;

/**
 * Test-Klasse für UseCase2BeteiligungNeu
 */
@Ignore("Einrichtung notwendig")
public class UseCase2BeteiligungNeuTest extends BaseTest {

  public UseCase2BeteiligungNeuTest() throws ClientInitializationException {
  }


  @Test
  public void nachrichtBeteiligung2PlanungBeteiligungNeuOK0411() throws Exception {

    final Date sendDate = new Date();
    //A1 sendet die Nachricht 'beteiligung2Planung.BeteiligungNeuOK.0411' und prüft ob an K1 übermittelt wurde
    // deliveryOptions.withCheckDeliveryStatus(DeliveryStatus.RECEIVED_IN_MSGBOX)
    DeliveryOptions deliveryOptions = new DeliveryOptions("urn:xbauleitplanung-beteiligung2planung:1.0",
        BusinessScenario.XBAULEITPLANUNG_DATA);
    deliveryOptions.withCheckDeliveryStatus(DeliveryStatus.RECEIVED_IN_MSGBOX);

    final Message message = createMessage("beteiligung2Planung.BeteiligungNeuOK.0411.xml");
    final MessageTransportReport transportReport = a1XtaClient.sendMessage(identifierK1, deliveryOptions, message);
    final String a1MessageId = transportReport.getMessageId();

    //Als K1 neue Nachrichten holen
    MessageFilter messageFilter = new MessageFilter().newEntries().withCount(500).withTimeFrom(sendDate);
    List<MessageTransportReport> messageTransportReportList = k1XtaClient.getMessageTransportReportList(messageFilter);
    assertThat(messageTransportReportList.size()).isGreaterThan(0);

    //Prüfen ob Nachricht enthalten ist
    //Achtung dies ist nur bei Verwendung des gelichen XTA-Brokers möglich. Ansonsten unterscheiden sich die MessageId's.
    //D.h. die Referenz MessageID (Author) -> MessageId (Reader) gibt es nicht.
    final String k1MessageId = getAndCheckMessageInMessageTransportList(a1MessageId, messageTransportReportList);
    assertThat(a1MessageId).isEqualTo(k1MessageId);

    //Als K1 die Nachricht abholen
    final GetMessageResult getMessageResult = k1XtaClient.getMessage(k1MessageId);
    assertThat(getMessageResult.getMessage().getData()).isEqualTo(message.getData());

    //Hier könnte von k1 die Nachricht lokal weiterverarbeitet werden
    //Nach erfolgter Verarbeitung muss die Nachricht quitiert werden.
    //K1 Bestätigt die Nachricht
    k1XtaClient.close(k1MessageId, getMessageResult.getRequestHandle());

    //Als A1 prüfen ob die Nachricht von K1 abgeholt wurde
    checkDeliveryStatus(a1XtaClient, a1MessageId, DeliveryStatus.RECEIPT_FROM_MSGBOX);
  }

  @Test
  public void nachrichtPlanung2BeteiligungBeteiligungNeu0401() throws Exception {

    //Als K1 senden der Nachricht 'planung2Beteiligung.BeteiligungNeu.0401'
    DeliveryOptions deliveryOptions = new DeliveryOptions("urn:xbauleitplanung-planung2beteiligung:1.0",
        BusinessScenario.XBAULEITPLANUNG_DATA);

    final Message message = createMessage("planung2Beteiligung.BeteiligungNeu.0401.xml");
    final MessageTransportReport transportReport = k1XtaClient.sendMessage(identifierA1, deliveryOptions, message);
    final String k1MessageId = transportReport.getMessageId();

    //Prüfen ob im Postfach an A1 übermittellt wurde.
    checkDeliveryStatus(k1XtaClient, k1MessageId, DeliveryStatus.RECEIVED_IN_MSGBOX);

    //Als A1 neue Nachrichten holen
    MessageFilter messageFilter = new MessageFilter().newEntries().withCount(500);
    List<MessageTransportReport> messageTransportReportList = a1XtaClient.getMessageTransportReportList(messageFilter);
    assertThat(messageTransportReportList.size()).isGreaterThan(0);

    //Prüfen ob Nachricht enthalten ist
    //Achtung dies ist nur bei Verwendung des gleichen XTA-Brokers möglich. Ansonsten unterscheiden sich die MessageId's.
    //D.h. die Referenz MessageID (Author) -> MessageId (Reader) gibt es nicht.
    String a1MessageId = getAndCheckMessageInMessageTransportList(k1MessageId, messageTransportReportList);
    assertThat(a1MessageId).isEqualTo(k1MessageId);

    //Als A1 die Nachricht abholen
    final GetMessageResult getMessageResult = a1XtaClient.getMessage(a1MessageId);
    assertThat(getMessageResult.getMessage().getData()).isEqualTo(message.getData());

    //Hier könnte von A1 die Nachricht lokal weiterverarbeitet werden
    //Nach erfolgter Verarbeitung muss die Nachricht quitiert werden.
    //A1 Bestätigt die Nachricht
    a1XtaClient.close(a1MessageId, getMessageResult.getRequestHandle());

    //Als K1 prüfen ob die Nachricht von A1 abgeholt wurde
    checkDeliveryStatus(k1XtaClient, k1MessageId, DeliveryStatus.RECEIPT_FROM_MSGBOX);
  }

}
