/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client;

import static org.assertj.core.api.Assertions.assertThat;

import de.xoev.xta.client.config.XtaClientSettings;
import de.xoev.xta.client.exception.ClientException;
import de.xoev.xta.client.model.DeliveryStatus;
import de.xoev.xta.client.model.Identifier;
import de.xoev.xta.client.model.Message;
import de.xoev.xta.client.model.MessageTransportReport;
import java.io.IOException;
import java.util.List;

import org.apache.commons.io.IOUtils;

/**
 * Basis Testklasse
 */
public abstract class BaseTest {

  Message createTestMessage() throws IOException {
    return createMessage("sample-message.xml");
  }

  Message createMessage(final String ressource) throws IOException {
    return new Message("application/xml",
        IOUtils.toByteArray(this.getClass().getClassLoader().getResourceAsStream("data/" + ressource))).withFilename(ressource);

  }

  protected static void checkDeliveryStatus(final XtaClient client, final String messageId, final DeliveryStatus status)
      throws Exception,
      InterruptedException {

    for (int i = 0; i < 5; i++) {
      final MessageTransportReport messageTransportReport = client.getMessageTransportReport(messageId);
      assertThat(messageTransportReport).isNotNull();
      if (messageTransportReport.getDeliveryStatus().ordinal() >= status.ordinal()) {
        return;
      }

      Thread.sleep(30 * 1000);
    }

    assertThat(true).isFalse();
  }

  protected static String getAndCheckMessageInMessageTransportList(String receiverId,
      List<MessageTransportReport> messageTransportReportList) {
    String msgId = null;
    for (final MessageTransportReport messageTransportReport : messageTransportReportList) {
      if (messageTransportReport.getMessageId().equals(receiverId)) {
        msgId = messageTransportReport.getMessageId();
        break;
      }
    }

    assertThat(msgId).isNotEmpty();
    return msgId;
  }

  protected XtaClient initializeClient(final XtaClientSettings settings) throws ClientException {
    return new XtaClient(XtaClientConfig.create(settings).withSoapLogging().build());
  }

  protected XtaClient initializeClient(final Identifier author, final XtaClientSettings settings) throws ClientException {
    return new XtaClient(XtaClientConfig.create(author, settings).withSoapLogging().build());
  }

}
