/**
 *     XTA-Client Java Library
 *     Copyright (C) 2021 Koordinierungsstelle für IT-Standards (KoSIT)
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.xoev.xta.client.encryption;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.xml.namespace.NamespaceContext;

/**
 * A NamespaceContext implementation for digital signatures
 */
public class DSNamespaceContext implements NamespaceContext {

  private Map<String, String> namespaceMap =
      new HashMap<String, String>();

  public DSNamespaceContext() {
    namespaceMap.put("ds", "http://www.w3.org/2000/09/xmldsig#");
    namespaceMap.put("dsig", "http://www.w3.org/2000/09/xmldsig#");
    namespaceMap.put("xenc", "http://www.w3.org/2001/04/xmlenc#");
  }

  public DSNamespaceContext(Map<String, String> namespaces) {
    this();
    namespaceMap.putAll(namespaces);
  }

  public String getNamespaceURI(String arg0) {
    return namespaceMap.get(arg0);
  }

  public void putPrefix(String prefix, String namespace) {
    namespaceMap.put(prefix, namespace);
  }

  public String getPrefix(String arg0) {
    for (String key : namespaceMap.keySet()) {
      String value = namespaceMap.get(key);
      if (value.equals(arg0)) {
        return key;
      }
    }
    return null;
  }

  public Iterator<String> getPrefixes(String arg0) {
    return namespaceMap.keySet().iterator();
  }
}
